# utils.py

import os
import platform

def linux():
    return platform.system() == 'Linux'

def windows():
    return platform.system() == 'Windows'

def macOS():
    return platform.system() == 'Darwin'

def GetLibSubFolderName():
    if windows():
        return 'windows'
    elif linux():
        return 'linux'
    elif macOS():
        return 'macOS'
    else:
        raise NameError('Os Type ', platform.system() + ' unknown')

def GetLibPath(libPath = ''):
    if libPath == '':
        libPath = os.path.join(os.path.dirname(os.path.abspath(__file__)),'lib')
    return os.path.join(libPath,GetLibSubFolderName())

def GetDataPath():
    return os.path.join(os.environ['HOME'],'data')

def GetLibFullName(baseName, libPath = ''):
    if windows():
        return os.path.join(GetLibPath(libPath), (baseName + '.dll'))
    else:
        return os.path.join(GetLibPath(libPath), ('lib' + baseName + '.so'))

def LoadLib(baseName, register = True, libPath = ''):
    import ctypes
    libPath = GetLibFullName(baseName, libPath)
    #print 'Loading library ', libPath
    loadedLib = ctypes.CDLL(libPath)
    if register:
        functionName = 'Register_Project_' + baseName
        # We allow for libraries which don't use xcpp and therefore don't have a registration
        # function
        if hasattr(loadedLib, functionName):
            #print 'calling ' + functionName
            loadedLib[functionName]()
    return loadedLib

# Called from __init__.py of python extension packages contains a lib folder with libraries
# to be loaded/registered.
#
# Typically a __init__.py looks like this:
#
#       import pyceda.utils
#       pyceda.utils.LoadLibraries(__file__)
#
# LoadLibraries only supports linux only at the moment.
#
# LoadLibraries has parameter 'file' which is expected to be the value of __file__ from
# the __init__.py of a module which wants to load/register all its libraries automatically.
#
# It is assumed the folder structure of the package resembles:
#
#   mypackage
#       +-- __init__.py
#       +-- lib
#           +-- linux
#           |   +-- libcat.so
#           |   +-- libdog.so
#           |   +-- libelephant.so
#           |   +-- libmonkey.so
#           +-- macOS
#           +-- windows
#
# In this case the basename is 'cat', 'dog', 'elephant' and 'monkey'

def FilenameIsLibrary(filename):
    if windows():
        return filename.endswith('.dll')
    else:
        return filename.startswith('lib') and filename.endswith('.so')

def GetBaseName(filename):
    if windows():
        # remove trailing '.dll'
        return filename[0:len(filename)-4]
    else:
        # remove leading 'lib' and trailing '.so'
        return filename[3:len(filename)-3]

def LoadLibraries(file):
    libPath = os.path.join(os.path.dirname(os.path.abspath(file)),'lib')
    directory = os.path.join(libPath, GetLibSubFolderName())
    for filename in os.listdir(directory):
        if FilenameIsLibrary(filename):
            fullPath = os.path.join(directory, filename)
            LoadLib(GetBaseName(filename), True, libPath)
